/* 
 * File:   DateTime.cpp
 * Author: Henrik Thulin
 * 
 * Created on den 5 april 2014, 00:26
 */

#include "DateTime.h"

DateTime::DateTime() {
}

DateTime::DateTime(const DateTime& orig) {
}

DateTime::~DateTime() {

}

DateTime::DateTime(DateTimeData dt) {
    this->SetDateTime(dt);
}

const unsigned char DaysInMonth[12] = {

    31, // Jan
    28, // Feb, leap year = 29
    31, // Mar
    30, // Apr
    31, // May
    30, // Jun
    31, // Jul
    31, // Aug
    30, // Sep
    31, // Oct
    30, // Nov
    31 // Dec
};

DateTime::DateTime(short year, char month, char day, char hour, char minute, char second) {


    this->dateTime.year = year;
    this->dateTime.month = month;
    this->dateTime.day = day;
    this->dateTime.hour = hour;
    this->dateTime.minute = minute;
    this->dateTime.second = second;
}

bool DateTime::IsLeap(DateTime::DateTimeData dt) {
    if (dt.year % 400 == 0)
        return 1;
    else if (dt.year % 100 == 0)
        return 0;
    else if (dt.year % 4 == 0)
        return 1;
    else
        return 0;
}

DateTime::DateTimeData DateTime::GetDateTime() {
    return this->dateTime;
}

void DateTime::SetDateTime(DateTimeData dt) {
    this->dateTime = dt;
}

unsigned char DateTime::GetDaysInMonth(DateTime::DateTimeData dt) {

    if (dt.month == 2 && DateTime::IsLeap(dt))
        return DaysInMonth[1] + 1;
    else
        return DaysInMonth[dt.month - 1];
}

// Sakamotos method. 0 = Sunday, 1 = Monday....

unsigned char DateTime::GetDayOfWeek(DateTime::DateTimeData dt) {

    int t[] = {0, 3, 2, 5, 0, 3, 5, 1, 4, 6, 2, 4};
    int y = dt.year;
    y -= (dt.month < 3);
    return (y + y / 4 - y / 100 + y / 400 + t[dt.month - 1] + dt.day) % 7;
}

void DateTime::AddDay() {

    this->dateTime.day++;

    if (this->dateTime.day > DateTime::GetDaysInMonth(GetDateTime())) {

        this->dateTime.month++;
        this->dateTime.day = 1;

        if (this->dateTime.month > 12) {
            this->dateTime.year++;
            this->dateTime.month = 1;
        }
    }
}

void DateTime::AddDays(int days) {

    while (days-- > 0)
        AddDay();
}

void DateTime::AddHour() {

    this->dateTime.hour++;

    if (this->dateTime.hour > 23) {
        AddDay();
        this->dateTime.hour = 0;
    }
}

void DateTime::AddHours(int hours) {

    while (hours-- > 0)
        AddHour();
}

bool DateTime::Equals(DateTime* dt) {

    if (this->dateTime.second != dt->dateTime.second) return false;
    if (this->dateTime.minute != dt->dateTime.minute) return false;
    if (this->dateTime.hour != dt->dateTime.hour) return false;
    if (this->dateTime.day != dt->dateTime.day) return false;
    if (this->dateTime.month != dt->dateTime.month) return false;
    if (this->dateTime.year != dt->dateTime.year) return false;

    return true;
}

void DateTime::Copy(DateTime* source) {

    this->dateTime = source->dateTime;
}

bool DateTime::IsSwedishSummerTime() {
    if (this->dateTime.month > 3 && this->dateTime.month < 9)
        return true;

    if (this->dateTime.month < 3 || this->dateTime.month > 9)
        return false;

    DateTimeData dt = this->GetDateTime();

    dt.day = GetDaysInMonth(this->GetDateTime());

    while (GetDayOfWeek(dt) != 0) dt.day--; // Find last sunday

    if (this->dateTime.month == 3)
        return this->dateTime.day > dt.day;

    if (this->dateTime.month == 9)
        return this->dateTime.day <= dt.day;

    return true;
}

