/*
 * File:   ds3232.cpp
 * Author: Rutger
 *
 * Created on den 22 januari 2014, 19:18
 */

#include <plib.h>
#include <time.h>
#include "ds3232.h"
#include "DateTime.h"

ds3232::ds3232() {
}

ds3232::ds3232(const ds3232& orig) {
}

ds3232::~ds3232() {
}

void I2CAck()
{
    unsigned int timeout = 0;
    while (I2C1STATbits.ACKSTAT != 0 && timeout++ < 0xffff) Nop();
}

void ds3232::SendData(unsigned char adr, unsigned char data) {
    StartI2C1();
    IdleI2C1();
    MasterWriteI2C1(0b11010000);
    IdleI2C1();
    I2CAck();
    MasterWriteI2C1(adr);
    IdleI2C1();
    I2CAck();
    MasterWriteI2C1(data);
    IdleI2C1();
    I2CAck();
    StopI2C1();
    IdleI2C1();
}

unsigned char GetData(unsigned char adr) {
    StartI2C1();
    IdleI2C1();

    MasterWriteI2C1(0b11010000);
    IdleI2C1();
    I2CAck();

    MasterWriteI2C1(adr);
    IdleI2C1();
    I2CAck();

    RestartI2C1();
    IdleI2C1();

    MasterWriteI2C1(0b11010001);
    IdleI2C1();
    I2CAck();

    unsigned char c;
    c = MasterReadI2C1();

    StopI2C1();
    IdleI2C1();

    return c;
}

void ds3232::Init(void) {
    OpenI2C1(I2C_ON | I2C_SLW_DIS | I2C_ACK_EN, (80000000 / 2 / 5000000) - 2);

    if (GetData(0x0e) != 0b00100000)
        SendData(0x0e, 0b00100000);

    if (GetData(0x0f) != 0x00)
        SendData(0x0f, 0b00000000);

}

void ds3232::SetDateTime(DateTime::DateTimeData dt) {
    SendData(0x06, ((dt.year / 10) << 4 | (dt.year % 10)));
    SendData(0x05, ((dt.month / 10) << 4 | (dt.month % 10)));
    SendData(0x04, ((dt.day / 10) << 4 | (dt.day % 10)));

    SendData(0x02, (dt.hour > 19 ? 0b00100000 : dt.hour > 9 ? 0b00010000 : 0) | (dt.hour % 10));
    SendData(0x01, ((dt.minute / 10) << 4) + (dt.minute % 10));
    SendData(0x00, ((dt.second / 10) << 4) + (dt.second % 10));
}

unsigned short ds3232::GetTemperature() {

    unsigned char status;
    unsigned char msb;
    unsigned char lsb;

    SendData(0x0f, 0b00110000);

    do {
    status = GetData(0x0f);
    }
    while (status & 0x04);

    msb = GetData(0x11);
    lsb = GetData(0x12);

    return ((unsigned short)msb << 8) | ((unsigned short)lsb >> 6);

    //return ((unsigned int)GetData(0x11) << 2) | ((unsigned int)GetData(0x12) >> 6);
}

DateTime::DateTimeData ds3232::GetDateTime(void) {

    DateTime::DateTimeData dt;

    dt.year = GetData(0x06);
    dt.month = GetData(0x05);
    dt.day = GetData(0x04);
    dt.hour = GetData(0x02);
    dt.minute = GetData(0x01);
    dt.second = GetData(0x00);

    dt.day = ((dt.day & 0b00110000) >> 4) * 10 + (dt.day & 0b00001111);
    dt.month = ((dt.month & 0b00010000) >> 4) * 10 + (dt.month & 0b00001111);
    dt.year = ((dt.year & 0b11110000) >> 4) * 10 + (dt.year & 0b00001111);


    dt.second = ((dt.second >> 4)*10) + (dt.second & 0b00001111);
    dt.minute = ((dt.minute >> 4)*10) + (dt.minute & 0b00001111);
    dt.hour = (dt.hour & 0b00001111) + (dt.hour & 0b00010000 ? 10 : 0) + (dt.hour & 0b00100000 ? 20 : 0);

    return dt;
}

